@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# -*- coding: utf-8-unix -*-
+# -*- coding: utf-8 -*-
 
 """
 gantt.py - version and date, see below
@@ -29,8 +29,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
 __author__ = 'Alexandre Norman (norman at xael.org)'
-__version__ = '0.3.0'
-__last_modification__ = '2015.01.08'
+__version__ = '0.3.1'
+__last_modification__ = '2015.01.09'
 
 import datetime
 import logging
@@ -92,7 +92,7 @@ def add_vacations(start_date, end_date=None):
 
 ############################################################################
 
-def _init_log_to_sysout(level=logging.INFO):
+def init_log_to_sysout(level=logging.INFO):
     """
     debugging facilities
     """
@@ -736,7 +736,7 @@ class Task(object):
                     fill="#F08000",
                     stroke=color,
                     stroke_width=1,
-                    opacity=0.25,
+                    opacity=0.35,
                 ))
 
         svg.add(svgwrite.text.Text(self.fullname, insert=((x+2)*mm, (y + 5)*mm), fill='black', stroke='black', stroke_width=0, font_family="Verdana", font_size="15"))
@@ -925,7 +925,12 @@ class Project(object):
                     opacity=0.8
                     ))
             vlines.add(svgwrite.text.Text('{2}.{0:02}/{1:02}'.format(jour.day, jour.month, cal[jour.weekday()][0]), insert=((x*10+1)*mm, 9*mm), fill='black', stroke='black', stroke_width=0, font_family="Verdana", font_size="8"))
+
             daytext = []
+
+            if jour.day == 1 and jour.month == 1:
+                vlines.add(svgwrite.text.Text('{0}'.format(jour.year), insert=((x*10+1)*mm, 3*mm), fill='#800000', stroke='#800000', stroke_width=0, font_family="Verdana", font_size="12"))
+            
             if jour.weekday() == 0:
                 daytext.append('W:{0:02}'.format(jour.isocalendar()[1]))
 
@@ -933,7 +938,7 @@ class Project(object):
                 daytext.append('{0}'.format(jour.strftime("%B")[:3]))
 
             if len(daytext) > 0:
-                vlines.add(svgwrite.text.Text(' / '.join(daytext), insert=((x*10+2)*mm, 6*mm), fill='black', stroke='black', stroke_width=0, font_family="Verdana", font_size="12"))
+                vlines.add(svgwrite.text.Text(' / '.join(daytext), insert=((x*10+1)*mm, 6*mm), fill='black', stroke='black', stroke_width=0, font_family="Verdana", font_size="12"))
 
         vlines.add(svgwrite.shapes.Line(start=(maxx*cm, 1*cm), end=(maxx*cm, (maxy+1)*cm)))
 
@@ -974,8 +979,11 @@ class Project(object):
             end_date = end
 
 
+        if start_date > end_date:
+            __LOG__.critical('start date {0} > end_date {1}'.format(start_date, end_date))
+            sys.exit(1)
+
         maxx = (end_date - start_date).days 
-        maxy = self.nb_elements()
 
         ldwg = svgwrite.container.Group()
     
@@ -1033,6 +1041,12 @@ class Project(object):
         else:
             end_date = end
 
+
+        if start_date > end_date:
+            __LOG__.critical('start date {0} > end_date {1}'.format(start_date, end_date))
+            sys.exit(1)
+
+
         if resources is None:
             resources = self.get_resources()
 
@@ -1063,17 +1077,20 @@ class Project(object):
             ress.add(svgwrite.text.Text('{0}'.format(r.fullname), insert=(3*mm, (nline*10+7)*mm), fill='black', stroke='white', stroke_width=0, font_family="Verdana", font_size="18"))
             ldwg.add(ress)
 
+            conflict_display_line = nline
             nline += 1
-            if not one_line_for_tasks:
-                conflict_display_line = nline
-            else:
-                conflict_display_line = nline - 1
+
+            # nline += 1
+            # if not one_line_for_tasks:
+            #     conflict_display_line = nline 
+            # else:
+            #     conflict_display_line = nline - 1
 
             # and add vacations on the calendar
             vac = svgwrite.container.Group()
             cday = start_date
             while cday <= end_date:
-                if not r.is_available(cday):
+                if cday.weekday() not in NOT_WORKED_DAYS and cday not in VACATIONS and not r.is_available(cday):
                      vac.add(svgwrite.shapes.Rect(
                             insert=(((cday - start_date).days * 10 + 1)*mm, ((conflict_display_line)*10+1)*mm),
                             size=(4*mm, 8*mm),
@@ -1090,27 +1107,27 @@ class Project(object):
             conflicts = svgwrite.container.Group()
             for t in self.get_tasks():
                 if t.get_resources() is not None and r in t.get_resources():
-                    if not one_line_for_tasks:
-                        nline += 1
-                        
                     psvg, void = t.svg(prev_y = nline, start=start_date, end=end_date, color=self.color)
                     if psvg is not None:
                         ldwg.add(psvg)
+                        if not one_line_for_tasks:
+                            nline += 1
                     
                     cday = t.start_date()
                     while cday <= t.end_date():
-                        if cday in affected_days:
+                        if cday in affected_days and cday.weekday() not in NOT_WORKED_DAYS  and cday not in VACATIONS:
                             conflicts_tasks.append({'resource':r.name, 'tasks':affected_days[cday], 'day':cday, 'task':t.name })
                             __LOG__.warning('** Conflict between tasks for {0} on date {1} tasks : {2} vs {3}'.format(r.name, cday, ",".join(affected_days[cday]), t.name))
 
-                            vac.add(svgwrite.shapes.Rect(
-                                    insert=(((cday - start_date).days * 10 + 1 + 4)*mm, ((conflict_display_line)*10+1)*mm),
-                                    size=(4*mm, 8*mm),
-                                    fill="#AA0000",
-                                    stroke="#AA0000",
-                                    stroke_width=1,
-                                    opacity=0.65,
-                                    ))
+                            if cday >= start_date and cday <= end_date:
+                                vac.add(svgwrite.shapes.Rect(
+                                        insert=(((cday - start_date).days * 10 + 1 + 4)*mm, ((conflict_display_line)*10+1)*mm),
+                                        size=(4*mm, 8*mm),
+                                        fill="#AA0000",
+                                        stroke="#AA0000",
+                                        stroke_width=1,
+                                        opacity=0.65,
+                                        ))
 
                         try:
                             affected_days[cday].append(t.name)
@@ -1123,13 +1140,15 @@ class Project(object):
             if not one_line_for_tasks:
                 ldwg.add(
                     svgwrite.shapes.Line(
-                        start=((0)*cm, (nline+1)*cm), 
-                        end=((maxx+1)*cm, (nline+1)*cm), 
+                        start=((0)*cm, (nline)*cm), 
+                        end=((maxx+1)*cm, (nline)*cm), 
                         stroke='black',
-                        stroke_dasharray='5,3',
                         ))
                 
-            nline += 1
+            # nline += 1
+            if one_line_for_tasks:
+                nline += 1
+
 
         dwg = svgwrite.Drawing(filename, debug=True)
         dwg.add(svgwrite.shapes.Rect(
@@ -1209,17 +1228,23 @@ class Project(object):
 
         fprj = svgwrite.container.Group()
         if self.name != "":
-            fprj.add(svgwrite.text.Text('{0}'.format(self.name), insert=((6*level+3)*mm, ((prev_y)*10+7)*mm), fill='black', stroke='white', stroke_width=0, font_family="Verdana", font_size="18"))
+            # if ((self.start_date() >= start and self.end_date() <= end) 
+            #     or (self.start_date() >= start and (self.end_date() <= end or self.start_date() <= end))) or level == 1: 
+            if ((self.start_date() >= start and self.end_date() <= end) 
+                or ((self.end_date() >=start and self.start_date() <= end))) or level == 1: 
+                fprj.add(svgwrite.text.Text('{0}'.format(self.name), insert=((6*level+3)*mm, ((prev_y)*10+7)*mm), fill='black', stroke='white', stroke_width=0, font_family="Verdana", font_size="18"))
+
+                fprj.add(svgwrite.shapes.Rect(
+                        insert=((6*level+0.8)*mm, (prev_y+0.5)*cm),
+                        size=(0.2*cm, ((cy-prev_y-1)+0.4)*cm),
+                        fill='purple',
+                        stroke='lightgray',
+                        stroke_width=0,
+                        opacity=0.5
+                        ))
+            else:
+                cy -= 1
 
-            fprj.add(svgwrite.shapes.Rect(
-                    insert=((6*level+0.8)*mm, (prev_y+0.5)*cm),
-                    size=(0.2*cm, ((cy-prev_y-1)+0.4)*cm),
-                    fill='purple',
-                    stroke='lightgray',
-                    stroke_width=0,
-                    opacity=0.5
-                    ))
-            
         fprj.add(prj)
 
         return (fprj, cy-prev_y)
@@ -1321,8 +1346,8 @@ if __name__ == '__main__':
     # non regression test
     doctest.testmod()
 else:
-    _init_log_to_sysout(level=logging.DEBUG)
-    #_init_log_to_sysout(level=logging.WARNING)
+    #init_log_to_sysout(level=logging.DEBUG)
+    init_log_to_sysout(level=logging.WARNING)
 
 
 #<EOF>######################################################################
