@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# -*- coding: utf-8-unix -*-
+# -*- coding: utf-8 -*-
 """
 org2gantt.py - version and date, see below
 
@@ -27,6 +27,7 @@ __version__ = '0.3.0'
 __last_modification__ = '2015.01.08'
 
 
+import copy
 import datetime
 import logging
 import os
@@ -56,7 +57,6 @@ def __show_version__(name, **kwargs):
     """
     Show version
     """
-    import os
     print("{0} version {1}".format(os.path.basename(name), __version__))
     return True
 
@@ -94,13 +94,104 @@ def _init_log_to_sysout(level=logging.INFO):
     return
 
 
+############################################################################
+
+def make_task_from_node(n, prop={}, prev_task=''):
+    """
+    """
+    __LOG__.debug('make_task_from_node ({0})'.format({'n':n.headline, 'prop':prop, 'prev_task':prev_task}))
+    gantt_code = ''
+
+    try:
+        name = n.properties['task_id'].strip()
+        if name == '':
+            name = str(uuid.uuid4()).replace('-', '_')
+    except KeyError:
+        name = str(uuid.uuid4()).replace('-', '_')
+    
+    if ' ' in name:
+        __LOG__.critical('** Space in task_id: [{0}]'.format(name))
+        sys.exit(1)
+    
+    fullname = n.headline.strip().replace("'", '_')
+    start = end = duration = None
+    if n.scheduled != '':
+        start = "{0}".format(_iso_date_to_datetime(str(n.scheduled)))
+    if n.deadline != '':
+        end = "{0}".format(_iso_date_to_datetime(str(n.deadline)))
+    if 'Effort' in n.properties:
+        duration = n.properties['Effort'].replace('d', '')
+
+    if 'BLOCKER' in n.properties and n.properties['BLOCKER'].strip() == 'previous-sibling':
+        depends_of = ['task_{0}'.format(prev_task)]
+    else:
+        try:
+            depends = n.properties['BLOCKER'].split()
+        except KeyError:
+            depends_of = None
+        else: # no exception raised
+            depends_of = []
+            for d in depends:
+                depends_of.append('task_{0}'.format(d))
+
+    if 'ordered'in prop and prop['ordered'] and prev_task is not None and prev_task != '':
+        depends_of = ['task_{0}'.format(prev_task)]
+
+    if depends_of is not None and len(depends_of) == 0:
+        depends_of = None
+    
+    try:
+        percentdone = n.properties['PercentDone']
+    except KeyError:
+        percentdone = None
+    
+    if n.todo == 'DONE':
+        if percentdone is not None or percentdone != '100':
+            __LOG__.warning('** Task [{0}] marked as done but PercentDone is set to {1}'.format(name, percentdone))
+        percentdone = 100
+
+    # Resources as tag
+    if len(n.tags) > 0:
+        ress = "{0}".format(["{0}".format(x) for x in n.tags.keys()]).replace("'", "")
+    # Resources as properties
+    elif 'allocate' in n.properties:
+        ress = "{0}".format(["{0}".format(x) for x in n.properties['allocate'].replace(",", " ").split()]).replace("'", "")
+    else:
+        try:
+            ress = prop['resources']
+        except KeyError:
+            ress = None
+        except TypeError:
+            ress = None
+
+
+    # get color from task properties
+    if 'color' in n.properties:
+        color = "'{0}'".format(n.properties['color'].strip())
+
+    # inherits color if defined
+    elif 'color' in prop and prop['color'] is not None and n.todo in prop['color'] and  prop['color'][n.todo] is not None:
+        color = "'{0}'".format(prop['color'][n.todo])
+        
+    else:
+        color = None
+
+    
+    gantt_code += "task_{0} = gantt.Task(name='{1}', start={2}, stop={6}, duration={3}, resources={4}, depends_of={5}, percent_done={7}, fullname='{8}', color={9})\n".format(name, name, start, duration, ress, str(depends_of).replace("'", ""), end, percentdone, fullname, color)
+    
+    
+    return (name, gantt_code)
+
+
 ############################################################################
 
 @clize.clize(
     alias = {
-        'org': ('o',),
         'debug': ('d',),
         'gantt': ('g',),
+        'start_date': ('s',),
+        'end_date': ('e',),
+        'today': ('t',),
         },
     extra = (
         clize.make_flag(
@@ -110,13 +201,19 @@ def _init_log_to_sysout(level=logging.INFO):
             ),
         )
     )
-def __main__(org, gantt='', debug=False):
+def __main__(org, gantt='', start_date='', end_date='', today='', debug=False):
     """
     org2gantt.py
     
     org: org-mode filename
 
     gantt: output python-gantt filename (default sysout)
+
+    start_date: force start date for output
+
+    end_date: force end date for output
+
+    today: force today date
     
     debug: debug
 
@@ -125,9 +222,19 @@ def __main__(org, gantt='', debug=False):
 
     Written by : Alexandre Norman <norman at xael.org>
     """
+
+    gantt_code = """#!/usr/bin/env python3
+# -*- coding: utf-8-unix -*-
+
+import datetime
+import gantt
+"""
+
     global __LOG__
     if debug:
         _init_log_to_sysout(logging.DEBUG)
+        gantt_code += "\nimport logging\ngantt.init_log_to_sysout(logging.DEBUG)\n"
+
     else:
         _init_log_to_sysout()
 
@@ -140,35 +247,55 @@ def __main__(org, gantt='', debug=False):
 
     __LOG__.debug('_analyse_nodes ({0})'.format({'nodes':nodes}))
 
-    gantt_code = """#!/usr/bin/env python3
-# -*- coding: utf-8-unix -*-
-
-import datetime
-import gantt
-"""
-
+    # Get all todo items
+    LISTE_TODOS = {'TODO':None, 'DONE':None}
+    with open(org) as f:
+        for line in f.readlines():
+            if line[:10] == '#+SEQ_TODO':
+                kwlist = re.findall('([A-Z]+)\(', line)
+                for kw in kwlist:
+                    LISTE_TODOS[kw] = None
 
     # Find CONFIGURATION in heading
     n_configuration = None
     for n in nodes:
-        if n.headline == "CONFIGURATION":
+        if n.headline.strip() == "CONFIGURATION":
             n_configuration = n
 
     planning_start_date = None
     planning_end_date = None
     planning_today_date = _iso_date_to_datetime(str(datetime.date.today()))
-    bar_color = None
+    my_today = datetime.date.today()
+    bar_color = {'TODO':'#FFFF90'}
     one_line_for_tasks = False
 
     # Generate code for configuration
     if n_configuration is not None:
-        if 'color' in n_configuration.properties:
-            bar_color = n_configuration.properties['color']
+        for t in LISTE_TODOS:
+            if 'color_{0}'.format(t) in n_configuration.properties:
+                bar_color[t] = n_configuration.properties['color_{0}'.format(t)].strip()
 
-        if 'one_line_for_tasks' in n_configuration.properties and n_configuration.properties['one_line_for_tasks'] == 't':
+
+        if 'one_line_for_tasks' in n_configuration.properties and n_configuration.properties['one_line_for_tasks'].strip() == 't':
              one_line_for_tasks = True
 
-        if 'start_date' in n_configuration.properties:
+        if today != '':
+            planning_today_date = _iso_date_to_datetime(today)
+            y, m, d = today.split('-')
+            my_today = datetime.date(int(y), int(m), int(d))
+
+        elif 'today' in n_configuration.properties:
+            dates = re.findall('[1-9][0-9]{3}-[0-9]{2}-[0-9]{2}', n_configuration.properties['today'])
+            if len(dates) == 1:
+                planning_today_date = _iso_date_to_datetime(dates[0])
+                y, m, d = dates[0].split('-')
+                my_today = datetime.date(int(y), int(m), int(d))
+
+        if start_date != '':
+            y, m, d = start_date.split('-')
+            planning_start_date = _iso_date_to_datetime(start_date)
+
+        elif 'start_date' in n_configuration.properties:
             # find date and use it
             dates = re.findall('[1-9][0-9]{3}-[0-9]{2}-[0-9]{2}', n_configuration.properties['start_date'])
             if len(dates) == 1:
@@ -181,12 +308,16 @@ import gantt
 
                 sign = -1*(sign=='-') + 1*(sign=='+')
                 if what == 'd':
-                    planning_start_date = _iso_date_to_datetime(str(datetime.date.today() + datetime.timedelta(days=qte*sign)))
+                    planning_start_date = _iso_date_to_datetime(str(my_today + datetime.timedelta(days=qte*sign)))
                 elif what == 'w':
-                    planning_start_date = _iso_date_to_datetime(str(datetime.date.today() + datetime.timedelta(weeks=qte*sign)))
+                    planning_start_date = _iso_date_to_datetime(str(my_today + datetime.timedelta(weeks=qte*sign)))
 
 
-        if 'end_date' in n_configuration.properties:
+        if end_date != '':
+            y, m, d = end_date.split('-')
+            planning_end_date = _iso_date_to_datetime(end_date)
+
+        elif 'end_date' in n_configuration.properties:
             # find date and use it
             dates = re.findall('[1-9][0-9]{3}-[0-9]{2}-[0-9]{2}', n_configuration.properties['end_date'])
             if len(dates) == 1:
@@ -199,18 +330,14 @@ import gantt
 
                 sign = -1*(sign=='-') + 1*(sign=='+')
                 if what == 'd':
-                    planning_end_date = _iso_date_to_datetime(str(datetime.date.today() + datetime.timedelta(days=qte*sign)))
-                elif what == 'w':
-                    planning_end_date = _iso_date_to_datetime(str(datetime.date.today() + datetime.timedelta(weeks=qte*sign)))
-                elif what == 'm':
-                    planning_end_date = _iso_date_to_datetime(str(datetime.date.today() + datetime.timedelta(month=qte*sign)))
-                elif what == 'y':
-                    planning_end_date = _iso_date_to_datetime(str(datetime.date.today() + datetime.timedelta(years=qte*sign)))
+                    planning_end_date = _iso_date_to_datetime(str(my_today + datetime.timedelta(days=qte*sign)))
+                elif what == 'w':                                 
+                    planning_end_date = _iso_date_to_datetime(str(my_today + datetime.timedelta(weeks=qte*sign)))
+                elif what == 'm':                                 
+                    planning_end_date = _iso_date_to_datetime(str(my_today + datetime.timedelta(month=qte*sign)))
+                elif what == 'y':                                 
+                    planning_end_date = _iso_date_to_datetime(str(my_today + datetime.timedelta(years=qte*sign)))
 
-        if 'today' in n_configuration.properties:
-            dates = re.findall('[1-9][0-9]{3}-[0-9]{2}-[0-9]{2}', n_configuration.properties['today'])
-            if len(dates) == 1:
-                planning_today_date = _iso_date_to_datetime(dates[0])
 
 
 
@@ -224,7 +351,7 @@ import gantt
             n_resources.append(n)
         elif found == True and n.level <= plevel:
             break
-        if found == False and n.headline == "RESOURCES":
+        if found == False and n.headline.strip() == "RESOURCES":
             found = True
             plevel = n.level
 
@@ -237,9 +364,12 @@ import gantt
     for nr in range(len(n_resources)):
         r = n_resources[nr]
 
-        rname = r.headline
-        rid = r.properties['resource_id'].strip()
-        
+        rname = r.headline.strip().replace("'","_")
+        try:
+            rid = r.properties['resource_id'].strip()
+        except KeyError:
+            rid = 'r_'+str(uuid.uuid4()).replace('-', '_')
+
         if rid in resources_id:
             __LOG__.critical('** Duplicate resource_id: [{0}]'.format(rid))
             sys.exit(1)
@@ -277,7 +407,7 @@ import gantt
                     gantt_code += "{0}.add_vacations(dfrom={1})\n".format(rid, _iso_date_to_datetime(start))
                 
             else:
-                if line != '' and not line.startswith(':'):
+                if line != '' and not line.strip().startswith(':'):
                     __LOG__.warning("Unknown resource line : {0}".format(line))
 
 
@@ -292,7 +422,7 @@ import gantt
     # Find VACATIONS in heading
     n_vacations = None
     for n in nodes:
-        if n.headline == "VACATIONS":
+        if n.headline.strip() == "VACATIONS":
             n_vacations = n
 
     # Generate code for vacations
@@ -316,10 +446,8 @@ import gantt
     # Generate code for Projects
     gantt_code += "\n#### Projects \n"
     # Mother of all
-    gantt_code += "project = gantt.Project(color='{0}')\n".format(bar_color)
+    gantt_code += "project = gantt.Project(color='{0}')\n".format(bar_color['TODO'])
 
-    cproject = 0
-    ctask = 0
     prj_found = False
     tasks_name = []
     # for inheriting project, ORDERED, color, resources
@@ -332,17 +460,16 @@ import gantt
         
         # it's a task / level 1
         if n.level == 1 \
-               and  not n.headline in ('RESOURCES', 'VACATIONS', 'CONFIGURATION') \
+               and  not n.headline.strip() in ('RESOURCES', 'VACATIONS', 'CONFIGURATION') \
                and 'no_gantt' not in n.tags \
-               and n.todo in ('TODO', 'STARTED', 'HOLD', 'DONE', 'WAITING'):
+               and n.todo in LISTE_TODOS:
 
             __LOG__.debug(' task / level 1')
 
             prop_inherits = []
-            prj_found = False
             prj_found = True
             prev_task = None
-            
+
             # Add task
             name, code = make_task_from_node(n)
 
@@ -359,13 +486,14 @@ import gantt
         # Not a task, it's a project
         # it should have children
         elif n.level >= 1 \
-                 and  not n.headline in ('RESOURCES', 'VACATIONS', 'CONFIGURATION') \
+                 and  not n.headline.strip() in ('RESOURCES', 'VACATIONS', 'CONFIGURATION') \
                  and 'no_gantt' not in n.tags \
-                 and not n.todo in ('TODO', 'STARTED', 'HOLD', 'DONE', 'WAITING'):
+                 and not n.todo in LISTE_TODOS:
 
             if n.level == 1:
+                __LOG__.debug('** cleanup prop_inherits')
                 prev_task = None
-            
+                prop_inherits = []
 
             if n.level > 1 and prj_found == False:
                 __LOG__.debug(' do not keep')
@@ -378,7 +506,7 @@ import gantt
 
             __LOG__.debug(' new project heading')
 
-            gantt_code += "###### Project {0} \n".format(n.headline)
+            gantt_code += "###### Project {0} \n".format(n.headline.strip())
 
             try:
                 name = n.properties['task_id'].strip()
@@ -388,7 +516,11 @@ import gantt
 
             __LOG__.debug('{0}'.format(prop_inherits))
 
-            gantt_code += "project_{0} = gantt.Project(name='{1}', color='{2}')\n".format(name, n.headline, bar_color)
+            if bar_color['TODO'] is not None:
+                gantt_code += "project_{0} = gantt.Project(name='{1}', color='{2}')\n".format(name, n.headline.strip().replace("'", '_'), bar_color['TODO'])
+            else:
+                gantt_code += "project_{0} = gantt.Project(name='{1}', color=None)\n".format(name, n.headline.strip().replace("'", '_'))
+                
             try:
                 gantt_code += "project_{0}.add_task(project_{1})\n".format(prop_inherits[-1]['project_id'], name)
             except KeyError:
@@ -409,37 +541,62 @@ import gantt
                     prev_task = None
                     ordered = False
 
+            color = copy.deepcopy(bar_color)
             # Inherits color            
             if 'color' in n.properties:
-                color = n.properties['color']
+                color['TODO'] = n.properties['color']
             else:
                 if len(prop_inherits) > 0:
-                    color = prop_inherits[-1]['color']
+                    color['TODO'] = prop_inherits[-1]['color']['TODO']
                 else:
-                    color = None
+                    color['TODO'] = bar_color['TODO']
 
-    
-            prop_inherits.append({'ordered':ordered, 'color':color, 'project_id':name})
+
+            # Inherits resources
+            # Resources as tag
+            if len(n.tags) > 0:
+                ress = "{0}".format(["{0}".format(x) for x in n.tags.keys()]).replace("'", "")
+                # Resources as properties
+            elif 'allocate' in n.properties:
+                ress = "{0}".format(["{0}".format(x) for x in n.properties['allocate'].replace(",", " ").split()]).replace("'", "")
+            else:
+                try:
+                    ress = prop_inherits[-1]['resources']
+                except KeyError:
+                    ress = None
+                except IndexError:
+                    ress = None
+
+
+            prop_inherits.append({'ordered':ordered, 'color':color, 'project_id':name, 'resources':ress})
             prj_found = True
 
+
+
         # It's a task
         elif n.level >= 1 \
                  and prj_found == True \
-                 and  not n.headline in ('RESOURCES', 'VACATIONS', 'CONFIGURATION') \
+                 and  not n.headline.strip() in ('RESOURCES', 'VACATIONS', 'CONFIGURATION') \
                  and 'no_gantt' not in n.tags \
-                 and n.todo in ('TODO', 'STARTED', 'HOLD', 'DONE', 'WAITING'):
+                 and n.todo in LISTE_TODOS:
 
-            __LOG__.debug(' new task under project')
+            __LOG__.debug(' new task under project {0}'.format(n.headline))
 
             if n.level == 1:
                 prev_task = None
-                
+                prop_inherits = []
+                __LOG__.debug(' clean prop_inherits')
 
+                
+            if n.level > 1 and len(prop_inherits) < n.level - 1:
+                __LOG__.critical('pb in structure : task "{0}" do not belong to a project but a task - possible inheritance problem'.format(n.headline))
+                
 
             if len(prop_inherits) >= n.level:
                 __LOG__.debug(' go one level up')
                 prop_inherits = prop_inherits[:-1]
 
+            __LOG__.debug(' bar_color {0}'.format(bar_color))
 
             # Add task
             if len(prop_inherits) > 0:
@@ -478,11 +635,6 @@ import gantt
 
     # Full project
     gantt_code += "\n#### Outputs \n"
-    #gantt_code += "project_0 = gantt.Project(color='{0}')\n".format(bar_color)
-    # for i in range(1, cproject + 1):
-    #     gantt_code += "project_{0}.make_svg_for_tasks(filename='project_{0}.svg', today={1}, start={2}, end={3})\n".format(i, planning_today_date, planning_start_date, planning_end_date)
-    #     gantt_code += "project_{0}.make_svg_for_resources(filename='project_{0}_resources.svg', today={1}, start={2}, end={3}, one_line_for_tasks={4})\n".format(i, planning_today_date, planning_start_date, planning_end_date, one_line_for_tasks)
-        #gantt_code += "project_0.add_task(project_{0})\n".format(i)
 
     gantt_code += "project.make_svg_for_tasks(filename='project.svg', today={0}, start={1}, end={2})\n".format(planning_today_date, planning_start_date, planning_end_date)
     gantt_code += "project.make_svg_for_resources(filename='project_resources.svg', today={0}, start={1}, end={2}, one_line_for_tasks={3})\n".format(planning_today_date, planning_start_date, planning_end_date, one_line_for_tasks)
@@ -506,83 +658,6 @@ import gantt
 
 ############################################################################
 
-def make_task_from_node(n, prop={}, prev_task=''):
-    """
-    """
-    gantt_code = ""
-
-    try:
-        name = n.properties['task_id'].strip()
-    except KeyError:
-        name = str(uuid.uuid4()).replace('-', '_')
-    
-    if ' ' in name:
-        __LOG__.critical('** Space in task_id: [{0}]'.format(name))
-        sys.exit(1)
-    
-    fullname = n.headline
-    start = end = duration = None
-    if n.scheduled != '':
-        start = "{0}".format(_iso_date_to_datetime(str(n.scheduled)))
-    if n.deadline != '':
-        end = "{0}".format(_iso_date_to_datetime(str(n.deadline)))
-    if 'Effort' in n.properties:
-        duration = n.properties['Effort'].replace('d', '')
-    
-    try:
-        depends = n.properties['BLOCKER'].split()
-    except KeyError:
-        depends_of = None
-    else: # no exception raised
-        depends_of = []
-        for d in depends:
-            depends_of.append('task_{0}'.format(d))
-
-    if 'ordered'in prop and prop['ordered'] and prev_task is not None and prev_task != '':
-        depends_of = ['task_{0}'.format(prev_task)]
-
-    
-    try:
-        percentdone = n.properties['PercentDone']
-    except KeyError:
-        percentdone = None
-    
-    if n.todo == 'DONE':
-        if percentdone is not None:
-            __LOG__.warning('** Task [{0}] marked as done but PercentDone is set to {1}'.format(name, percentdone))
-        percentdone = 100
-    
-    # Resources as tag
-    if len(n.tags) > 0:
-        ress = "{0}".format(["{0}".format(x) for x in n.tags.keys()]).replace("'", "")
-    # Resources as properties
-    elif 'resource_id' in n.properties:
-        ress = "{0}".format(["{0}".format(x) for x in n.properties['resource_id'].split()]).replace("'", "")
-    else:
-        ress = None
-
-
-    # get color from task properties
-    if 'color' in n.properties:
-        color = "'{0}'".format(n.properties['color'])
-    # inherits color if defined 
-    elif 'color' in prop and prop['color'] is not None:
-        color = "'{0}'".format(prop['color'])
-    else:
-        color = None
-
-    
-    gantt_code += "task_{0} = gantt.Task(name='{1}', start={2}, stop={6}, duration={3}, resources={4}, depends_of={5}, percent_done={7}, fullname='{8}', color={9})\n".format(name, name, start, duration, ress, str(depends_of).replace("'", ""), end, percentdone, fullname, color)
-    
-    
-    return (name, gantt_code)
-
-############################################################################
-
-
-
-
-__all__ = ['org2gantt']
 
 
 # MAIN -------------------
